import logging
import os
from argparse import ArgumentParser
from pathlib import Path

import requests

from repo_cli.commands.base import SubCommandBase

logger = logging.getLogger("repo_cli")


class SubCommand(SubCommandBase):
    name = "download"

    def main(self):
        if self.args.channel and self.args.notebook:
            self.download_notebook(
                self.args.channel,
                self.args.notebook,
                self.args.filename if self.args.filename else None,
            )
        else:
            logger.info("Not enough parameters to identify download file")
            raise NotImplementedError()

    def download_notebook(
        self, channel: str, notebook_name: str, filename: os.path = None
    ):
        url = self.api.get_notebook_download_url(channel, notebook_name)
        r = requests.get(url, allow_redirects=True)

        open(self.get_filename(notebook_name, filename), "wb").write(r.content)

    @staticmethod
    def get_filename(notebook_name: str, filename: str = None):
        if filename:
            file = Path(filename)
            if not file.is_file():
                return os.path.abspath(filename)
            else:
                logger.error(
                    f"File {filename} already exists. Please specify a different filename."
                )
                raise FileExistsError
        filename = notebook_name + ".ipynb"
        return os.path.join(os.getcwd(), filename)

    def add_parser(self, subparsers: ArgumentParser):
        self.subparser = subparsers.add_parser(
            "download", help="Help string", description=__doc__
        )

        self.subparser.add_argument("--channel", type=str, help="Channel name")

        self.subparser.add_argument("--notebook", type=str, help="Notebook name")

        self.subparser.add_argument(
            "--filename",
            type=str,
            required=False,
            default=None,
            help="filename to save the report. Can contain full path for report to be saved in a different directory",
        )

        self.subparser.set_defaults(main=self.main)



#!/usr/bin/env python

import os
import sys

from platformdirs import site_data_dir, user_cache_dir, user_data_dir, user_log_dir

PY3 = sys.version_info[0] == 3

if PY3:
    unicode = str


class EnvAppDirs(object):
    def __init__(self, appname, appauthor, root_path):
        self.appname = appname
        self.appauthor = appauthor
        self.root_path = root_path

    @property
    def user_data_dir(self):
        return os.path.join(self.root_path, "data")

    @property
    def site_data_dir(self):
        return os.path.join(self.root_path, "data")

    @property
    def user_cache_dir(self):
        return os.path.join(self.root_path, "cache")

    @property
    def user_log_dir(self):
        return os.path.join(self.root_path, "log")


class AppDirs(object):
    """Convenience wrapper for getting application dirs."""

    def __init__(self, appname, appauthor, version=None, roaming=False):
        self.appname = appname
        self.appauthor = appauthor
        self.version = version
        self.roaming = roaming

    @property
    def user_data_dir(self):
        return user_data_dir(
            self.appname, self.appauthor, version=self.version, roaming=self.roaming
        )

    @property
    def site_data_dir(self):
        return site_data_dir(self.appname, self.appauthor, version=self.version)

    @property
    def user_cache_dir(self):
        return user_cache_dir(self.appname, self.appauthor, version=self.version)

    @property
    def user_log_dir(self):
        return user_log_dir(self.appname, self.appauthor, version=self.version)


def _get_win_folder_from_registry(csidl_name):
    """This is a fallback technique at best. I'm not sure if using the
    registry for this guarantees us the correct answer for all CSIDL_*
    names.
    """
    import _winreg

    shell_folder_name = {
        "CSIDL_APPDATA": "AppData",
        "CSIDL_COMMON_APPDATA": "Common AppData",
        "CSIDL_LOCAL_APPDATA": "Local AppData",
    }[csidl_name]

    key = _winreg.OpenKey(
        _winreg.HKEY_CURRENT_USER,
        r"Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders",
    )
    dir, type = _winreg.QueryValueEx(key, shell_folder_name)
    return dir


def _get_win_folder_with_pywin32(csidl_name):
    from win32com.shell import shell, shellcon

    dir = shell.SHGetFolderPath(0, getattr(shellcon, csidl_name), 0, 0)
    # Try to make this a unicode path because SHGetFolderPath does
    # not return unicode strings when there is unicode data in the
    # path.
    try:
        dir = unicode(dir)

        # Downgrade to short path name if have highbit chars. See
        # <http://bugs.activestate.com/show_bug.cgi?id=85099>.
        has_high_char = False
        for c in dir:
            if ord(c) > 255:
                has_high_char = True
                break
        if has_high_char:
            try:
                import win32api

                dir = win32api.GetShortPathName(dir)
            except ImportError:
                pass
    except UnicodeError:
        pass
    return dir


def _get_win_folder_with_ctypes(csidl_name):
    import ctypes

    csidl_const = {
        "CSIDL_APPDATA": 26,
        "CSIDL_COMMON_APPDATA": 35,
        "CSIDL_LOCAL_APPDATA": 28,
    }[csidl_name]

    buf = ctypes.create_unicode_buffer(1024)
    ctypes.windll.shell32.SHGetFolderPathW(None, csidl_const, None, 0, buf)

    # Downgrade to short path name if have highbit chars. See
    # <http://bugs.activestate.com/show_bug.cgi?id=85099>.
    has_high_char = False
    for c in buf:
        if ord(c) > 255:
            has_high_char = True
            break
    if has_high_char:
        buf2 = ctypes.create_unicode_buffer(1024)
        if ctypes.windll.kernel32.GetShortPathNameW(buf.value, buf2, 1024):
            buf = buf2

    return buf.value


if sys.platform == "win32":
    try:
        import win32com.shell

        _get_win_folder = _get_win_folder_with_pywin32
    except ImportError:
        try:
            import ctypes

            _get_win_folder = _get_win_folder_with_ctypes
        except ImportError:
            _get_win_folder = _get_win_folder_from_registry


# ---- self test code

if __name__ == "__main__":
    appname = "MyApp"
    appauthor = "MyCompany"

    props = ("user_data_dir", "site_data_dir", "user_cache_dir", "user_log_dir")

    print("-- app dirs (without optional 'version')")
    dirs = AppDirs(appname, appauthor, version="1.0")
    for prop in props:
        print("%s: %s" % (prop, getattr(dirs, prop)))

    print("\n-- app dirs (with optional 'version')")
    dirs = AppDirs(appname, appauthor)
    for prop in props:
        print("%s: %s" % (prop, getattr(dirs, prop)))



"""
Download artifact download reports from anaconda server

conda repo report

This command will download a report of all artifact downloads from the anaconda server and save it in the current working directory.
If you want to chane the download directory, use the --filename option.
"""
import datetime
import json
import logging
import os
from pathlib import Path

from ..utils.validators import check_date_format
from .base import SubCommandBase

logger = logging.getLogger("repo_cli")


class SubCommand(SubCommandBase):
    name = "report"

    def main(self):
        self.download_report(
            self.args.date_from,
            self.args.date_to,
            self.args.file_type,
            self.args.user_names,
            self.args.channels,
            self.args.filename if self.args.filename else None,
        )

    def download_report(
        self, date_from, date_to, file_type, usernames, channels, filename=None
    ):
        if date_from > date_to:
            raise ValueError(
                f"Invalid date range: The starting date {date_from} must be earlier than the ending date {date_to}."
            )

        usernames = usernames.split(",") if usernames else None
        channels = channels.split(",") if channels else None

        self.validate_channels(channels)

        logger.info("Downloading report for %s to %s" % (date_from, date_to))
        report_data = self.api.get_report(
            date_from, date_to, usernames, channels, file_type
        )

        if not report_data:
            logger.error("No report available for the given dates")
            return

        filepath = Path(self.get_filename(date_from, date_to, file_type, filename))
        logger.info(f"Saving report to {filepath}")
        if file_type == "json":
            filepath.write_text(json.dumps(report_data), encoding="utf-8")
        elif file_type == "csv":
            filepath.write_text(report_data, encoding="utf-8")

    @staticmethod
    def get_filename(date_from, date_to, file_type, filename=None):
        if filename:
            file = Path(filename)
            if not file.is_file():
                return os.path.abspath(filename)
            else:
                logger.error(
                    f"File {filename} already exists. Please specify a different filename."
                )
                raise FileExistsError
        filename = f"artifact_download_report_{date_from}_{date_to}.{file_type}"
        return os.path.join(os.getcwd(), filename)

    def validate_channels(self, channels):
        if not channels:
            return True

        all_channels = [
            channel["name"]
            for channel in self.api.list_channels()["items"]
            if channel["name"]
        ]

        for channel in channels:
            if channel not in all_channels:
                raise ValueError(f"Channel {channel} does not exist")

    def add_parser(self, subparsers):
        self.subparser = subparsers.add_parser(
            "report", help="download reports", description=__doc__
        )

        self.subparser.add_argument(
            "--date_from",
            type=check_date_format,
            default=None,
            help="starting date YYYY-mm-dd for the report",
            required=True,
        )

        self.subparser.add_argument(
            "--date_to",
            type=check_date_format,
            default=datetime.datetime.now().strftime("%Y-%m-%d"),
            help="end date YYYY-mm-dd for the report",
        )

        self.subparser.add_argument(
            "--file-type",
            type=str,
            default="json",
            choices=["json", "csv"],
            help="format of the report",
        )

        self.subparser.add_argument(
            "--user_names",
            type=str,
            help="comma separated list of users",
        )

        self.subparser.add_argument(
            "--channels",
            type=str,
            help="comma separated list of channels",
        )

        self.subparser.add_argument(
            "--filename",
            type=str,
            help="filename to save the report. Can contain full path for report to be saved in a different directory",
        )

        self.subparser.set_defaults(main=self.main)



import sys

from repo_cli import errors
from repo_cli.commands.base import SubCommandBase
from repo_cli.utils import bool_input


class BulkActionCommand(SubCommandBase):
    name = None

    def main(self):
        for spec in self.args.specs:
            try:
                if spec._filename:
                    self.exec_bulk_action(
                        spec.channel,
                        self.args.family,
                        spec.package,
                        spec.version,
                        spec.filename,
                        spec,
                    )
                elif spec._version:
                    self.exec_bulk_action(
                        spec.channel,
                        self.args.family,
                        spec.package,
                        spec.version,
                        spec=spec,
                    )
                elif spec._package:
                    self.exec_bulk_action(
                        spec.channel, self.args.family, spec.package, spec=spec
                    )
                else:
                    self.log.error("Invalid package specification: %s", spec)

            except errors.NotFound:
                if self.args.force:
                    self.log.warning("", exc_info=True)
                    continue
                else:
                    raise

    def exec_bulk_action(
        self, channel, family, artifact, version=None, filename=None, spec=None
    ):
        base_item = {
            "name": artifact,
            "family": family,
        }

        target_description = ""
        if hasattr(self.args, "destination"):
            target_channel = self.args.destination
            if not target_channel:
                # destination channel not specified.. we need to get the user default channel and use it
                pass
            if target_channel:
                target_description = "to channel %s " % target_channel
        items = []
        if version or filename:
            packages, total_count = self.api.get_channel_artifacts_files(
                channel, family, artifact, version, filename
            )

            if not packages:
                self.log.warning(
                    "No files matches were found for the provided spec: %s\n" % (spec)
                )
                return

            files_descr = []
            for filep in packages:
                files_descr.append(
                    "PACKAGE: {name}:{version}-{ckey}; PLATFORM: {platform}; FILENAME: {fn}".format(
                        **filep
                    )
                )
                item = dict(base_item)
                item["ckey"] = filep["ckey"]
                items.append(item)

            affected_files = "\n".join(files_descr)

            msg = (
                "Are you sure you want to %s the package release %s %s? The following "
                "will be affected: \n\n %s\n\nConfirm?"
                % (self.name, target_description, spec, affected_files)
            )
        else:
            msg = "Conform action %s on spec %s ? (and all data with it?)" % (
                self.name,
                spec,
            )
            items = [base_item]
        force = getattr(self.args, "force", False)
        if force or bool_input(msg, False):
            self.api.channel_artifacts_bulk_actions(
                channel, self.name, items, target_channel=target_channel
            )
            self.log.info("%s action successful\n" % self.name)
        else:
            self.log.info("%s action not executed\n" % self.name)

    def add_parser(self, subparsers):
        raise NotImplementedError



"""
Manage your Anaconda repository channels.
"""

from __future__ import print_function, unicode_literals

import argparse
import logging
import socket
import sys
from argparse import FileType
from datetime import datetime, timedelta

from .. import errors
from .base import SubCommandBase

logger = logging.getLogger("repo_cli")


class SubCommand(SubCommandBase):
    name = "auth"

    def main(self):
        if self.args.info:
            self.show_token_info()
        elif self.args.list:
            self.show_tokens()
            return
        elif self.args.remove:
            self.remove_tokens(self.args.remove)
            return
        elif self.args.list_scopes:
            self.show_scopes()
        elif self.args.create:
            self.create_token(self.args.name, self.args.scopes, self.args.max_age)

    def create_token(self, name, scopes, max_age):
        # for this one we need to ask with the JWT...
        if not self.api._jwt:
            self.parent.auth_manager.interactive_get_token()

        if not self.api._jwt:
            self.log.info("Failed to authenticate...")
            return
        if not scopes:
            self.log.warning(
                "You have not specified the scope of this token with the '--scopes' argument."
            )
            self.log.warning(
                "This token will grant full access to %s's account"
                % (self.parent.auth_manager.username)
            )
            self.log.warning(
                "Use the --list-scopes option to see a listing of your options"
            )
            scopes = self.api.get_scopes()

        expiration = str(datetime.now().date() + timedelta(seconds=max_age))
        data = self.api.create_user_token(
            name, expiration, scopes=scopes, resources=None
        )
        self.log.info(
            "Token {0} succesfully created with id: {1}".format(
                data["token"], data["id"]
            )
        )

    def show_scopes(self):
        scopes = self.api.get_scopes()
        self.log.info("")
        self.log.info("Available scopes:")
        self.log.info("")
        self.log.info(",".join(sorted(scopes)))
        self.log.info("")

    def remove_tokens(self, tokens):
        for token in tokens:
            self.api.remove_user_token(token)
            self.log.info("Token %s successfully removed." % token)

    def show_token_info(self):
        if not self.api._access_token:
            raise errors.Unauthorized()
        data = self.api.get_token_info()
        self.log.info("")
        self.log.info("Token: %s" % self.api._access_token)
        self.log.info("Expiration: %s" % data.get("expires_at", "n/a"))

    def show_tokens(self):
        # for this one we need to ask with the JWT...
        if not self.api._jwt:
            token = self.parent.auth_manager.login()

        if not self.api._jwt:
            self.log.info("Failed to authenticate...")
            return

        data = self.api.get_user_tokens()

        if data:
            self.log.info("")
            self.log.info("Tokens count: %s" % len(data["items"]))

            for token in data["items"]:
                self.log.info("-----------------")
                self.log.info("id: %s" % token["id"])
                self.log.info("name: %s" % token["name"])
                self.log.info("created_at: %s" % token["created_at"])
                self.log.info("expires_at: %s" % token["expires_at"])
                self.log.info("type: %s" % token["type"])
                self.log.info("-----------------")
            self.log.info("")

    def add_parser(self, subparsers):
        description = "Manage Authorization Tokens"
        parser = subparsers.add_parser(
            self.name,
            help=description,
            description=description,
            epilog=__doc__,
            formatter_class=argparse.RawDescriptionHelpFormatter,
        )

        parser.add_argument(
            "-n",
            "--name",
            default="conda_repo:%s" % (socket.gethostname()),
            help="A unique name so you can identify this token later.",
        )

        # parser.add_argument('-o', '--org', '--organization', help='Set the token owner (must be an organization)',
        #                     dest='organization')

        g = parser.add_argument_group(
            "token creation arguments",
            "These arguments are only valid with the `--create` action",
        )

        # g.add_argument('--strength', choices=['strong', 'weak'], default='strong', dest='strength')
        # g.add_argument('--strong', action='store_const', const='strong', dest='strength',
        #                help='Create a longer token (default)')
        # g.add_argument('-w', '--weak', action='store_const', const='weak', dest='strength',
        #                help='Create a shorter token')

        # g.add_argument('--url', default='http://anaconda.org',
        #                help='The url of the application that will use this token')
        age_in_days = 30
        default_age = 86400 * age_in_days  # 30 days
        g.add_argument(
            "--max-age",
            type=int,
            default=default_age,
            help="The maximum age in seconds that this token will be valid for (default value is"
            " %s that equals to %s days" % (default_age, age_in_days),
        )
        g.add_argument(
            "-s",
            "--scopes",
            action="append",
            help=(
                "Scopes for token. For example if you want to limit this token to conda "
                'downloads only you would use --scopes "repo artifact:download"'
            ),
            default=[],
        )

        g.add_argument("--out", default=sys.stdout, type=FileType("w"))

        group = parser.add_argument_group("actions")
        group = group.add_mutually_exclusive_group(required=True)
        group.add_argument(
            "-x",
            "--list-scopes",
            action="store_true",
            help="list all authentication scopes",
        )
        group.add_argument(
            "-l",
            "--list",
            action="store_true",
            help="list all user authentication tokens",
        )
        group.add_argument(
            "-r",
            "--remove",
            metavar="ID",
            nargs="+",
            help="remove authentication tokens",
        )
        group.add_argument(
            "-c", "--create", action="store_true", help="Create an authentication token"
        )
        group.add_argument(
            "-i",
            "--info",
            "--current-info",
            dest="info",
            action="store_true",
            help="Show information about the current authentication token",
        )

        parser.set_defaults(main=self.main)



import unittest
from unittest.mock import patch

from repo_cli.commands.channel import SubCommand


@patch("repo_cli.commands.base.SubCommandBase")
class Test(unittest.TestCase):
    subcommand = None

    def setup(self, mock_command_base):
        self.subcommand = SubCommand(mock_command_base)
        self.subcommand.args.create = None
        self.subcommand.args.remove = None
        self.subcommand.args.list = None
        self.subcommand.args.list_files = None
        self.subcommand.args.list_packages = None
        self.subcommand.args.remove = None
        self.subcommand.args.show = None
        self.subcommand.args.history = None
        self.subcommand.args.lock = None
        self.subcommand.args.soft_lock = None
        self.subcommand.args.unlock = None
        self.subcommand.args.freeze = None
        self.subcommand.args.unfreeze = None

    def test_lock_channel(self, mock_command_base):
        mock_command_base.api.update_channel.return_value = ""
        self.setup(mock_command_base)
        self.subcommand.args.lock = "test_channel"
        self.subcommand.main()
        mock_command_base.api.update_channel.assert_called_once()
        mock_command_base.api.update_channel.assert_called_with(
            "test_channel",
            privacy="private",
            success_message="Channel test_channel is now locked",
        )

    def test_unlock_channel(self, mock_command_base):
        mock_command_base.api.update_channel.return_value = ""
        self.setup(mock_command_base)
        self.subcommand.args.unlock = "test_channel"
        self.subcommand.main()
        mock_command_base.api.update_channel.assert_called_once()
        mock_command_base.api.update_channel.assert_called_with(
            "test_channel",
            privacy="public",
            success_message="Channel test_channel is now unlocked",
        )

    def test_soft_lock_channel(self, mock_command_base):
        mock_command_base.api.update_channel.return_value = ""
        self.setup(mock_command_base)
        self.subcommand.args.soft_lock = "test_channel"
        self.subcommand.main()
        mock_command_base.api.update_channel.assert_called_once()
        mock_command_base.api.update_channel.assert_called_with(
            "test_channel",
            privacy="authenticated",
            success_message="Channel test_channel is now soft-locked",
        )



# -*- coding: utf-8 -*-
# Copyright (C) 2019 Anaconda, Inc
import logging
from pathlib import Path
from random import randint
from re import findall
from unittest.mock import Mock

import pytest
import responses
import yaml

from repo_cli import errors, main
from repo_cli.utils import config
from repo_cli.utils.api import RepoApi

from .integration.testcase import load_mocks

here = Path(__file__).parent
logging.basicConfig(level=logging.INFO)

log = logging.getLogger("repo_cli")
log.setLevel(logging.DEBUG)

default_config = config.get_config()
test_site = default_config.get("default_site")
test_url = default_config.get("url")

DEFAULT_PACKAGE_PATH = str(here / "data/numpy-1.15.4-py37hacdab7b_0.tar.bz2")

try:
    with open(str(here.joinpath("test_config.yaml")), "r") as fp:
        _auth_config = yaml.load(fp, yaml.Loader)
        default_username = _auth_config["user1"]["username"]
        default_pwd = _auth_config["user1"]["password"]

        username_2 = _auth_config["user2"]["username"]
        password_2 = _auth_config["user2"]["password"]
except FileNotFoundError:
    print(
        "Configuration file test_config.yaml not found. Cannot run integration tests without users "
        "information. Please create your configuration file in the following format:"
        "user1:"
        "  password: password"
        "  username: username1"
        "user2:"
        "  password: password"
        "  username: username2"
    )

TOKEN_USER_1 = None
TOKEN_USER_2 = None


@pytest.fixture
def default_user(auth_config):
    return auth_config["user1"]


@pytest.fixture
def default_username(auth_config):
    return auth_config["user1"]["username"]


@pytest.fixture
def auth_config():
    return _auth_config


@pytest.fixture
def token_user_1(default_username):
    global TOKEN_USER_1

    with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
        load_mocks(rsps)
        if TOKEN_USER_1:
            return TOKEN_USER_1

        token = config.load_token(test_site)
        if not token:
            main.main(
                [
                    "login",
                    "--username=%s" % default_username,
                    "--password=%s" % default_pwd,
                ]
            )
            token = config.load_token(test_site)

        TOKEN_USER_1 = token
        return token


@pytest.fixture
def token_user_2():
    global TOKEN_USER_2

    with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
        load_mocks(rsps)
        if TOKEN_USER_2:
            return TOKEN_USER_2

        args = Mock(site=test_site)
        main.main(["logout"])
        main.main(["login", "--username=%s" % username_2, "--password=%s" % password_2])
        token = config.load_token(test_site)
        args = Mock(site=test_site)
        config.remove_token(args)

        # now we need to store the  user 1 token back so it can be used as "default" user
        config.store_token(TOKEN_USER_1, args)
        TOKEN_USER_2 = token

        return token


@pytest.fixture
def new_channel(token_user_1):
    channel_name = "test_channel_1234"
    with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
        load_mocks(rsps)
        main.main(["-t", token_user_1, "channel", "--create", channel_name])

        yield channel_name

        main.main(["-t", token_user_1, "channel", "--remove", channel_name])


@pytest.fixture
def new_channel_with_package(token_user_1, new_channel):
    main.main(["-t", token_user_1, "upload", "-c", new_channel, DEFAULT_PACKAGE_PATH])

    return new_channel


@pytest.fixture
def many_channels(token_user_1):
    channel_names = []

    for x in range(5):
        just_a_number = randint(0, 9999)
        channel_name = f"test_channel_0{just_a_number}"
        main.main(["-t", token_user_1, "channel", "--create", channel_name])
        channel_names.append(channel_name)

    yield channel_names

    for channel_name in channel_names:
        main.main(["-t", token_user_1, "channel", "--remove", channel_name])


@pytest.fixture
def new_subchannel(token_user_1, new_channel):
    subchannel_name = "test_channel_1234"
    with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
        load_mocks(rsps)
        main.main(
            [
                "-t",
                token_user_1,
                "channel",
                "--create",
                new_channel + "/" + subchannel_name,
            ]
        )

        yield new_channel + "/" + subchannel_name

        main.main(
            [
                "-t",
                token_user_1,
                "channel",
                "--remove",
                new_channel + "/" + subchannel_name,
            ]
        )


@pytest.fixture
def new_subchannel_with_package(token_user_1, new_subchannel):
    main.main(
        ["-t", token_user_1, "upload", "-c", new_subchannel, DEFAULT_PACKAGE_PATH]
    )

    return new_subchannel


@pytest.fixture
def package_path():
    return DEFAULT_PACKAGE_PATH


@pytest.fixture
def package_filename():
    return DEFAULT_PACKAGE_PATH.split("/")[-1]


@pytest.fixture
def package_name():
    return DEFAULT_PACKAGE_PATH.split("/")[-1].split("-")[0]


@pytest.fixture
def package_version():
    return DEFAULT_PACKAGE_PATH.split("/")[-1].split("-")[1]


@pytest.fixture
def CVE_ids(caplog, token_user_1):
    main.main(["-t", token_user_1, "cves", "--list"])

    return findall(r"CVE-\d{4}-\d{4}", caplog.text)


# Trying to return a function to work around passing arguements to fixtures.
# It would be nice to be able to just say give me a (sub)channel with package(s)/file(s)
# TODO:figure out a way to pass the has_package param.
@pytest.fixture
def experimental_new_channel(token_user_1):
    def new_channel(has_package=False):
        just_a_number = randint(0, 9999)
        channel_name = "test_channel_0{}".format(just_a_number)
        main.main(["-t", token_user_1, "channel", "--create", channel_name])

        if has_package:
            main.main(
                ["-t", token_user_1, "upload", "-c", channel_name, DEFAULT_PACKAGE_PATH]
            )

        return channel_name

    channel_name = new_channel()
    yield channel_name

    main.main(["-t", token_user_1, "channel", "--remove", channel_name])


# patch methods to force the tokens into place when the CLI asks for additional login info
# todo: this causes token issues with tests that don't use it, see CBR-1594
def run_patched(self):
    default_username = _auth_config["user1"]["username"]
    default_pwd = _auth_config["user1"]["password"]

    try:
        token = get_token_patched(default_username, default_pwd, test_url)
        self.username = default_username
        self._access_token = token
        self.api._access_token = self._access_token
        self.api._jwt = token

    # If there is weirdness related to tokens disable this exception suppression.
    except AttributeError:
        pass

    try:
        try:
            if not hasattr(self.args, "main"):
                self.parser.error(
                    "A sub command must be given. "
                    "To show all available sub commands, run:\n\n\t conda repo -h\n"
                )

            return self.args.main()

        except errors.Unauthorized:
            self.log.info(
                "The action you are performing requires authentication, "
                "please sign in:"
            )
            self._access_token = self.auth_manager.login()
            return self.args.main(self)

    except errors.ShowHelp:
        self.args.sub_parser.print_help()
        if exit:
            raise SystemExit(1)
        else:
            return 1


def get_token_patched(default_username, default_pwd, test_url):
    token_api = RepoApi(test_url)

    return token_api.login(default_username, default_pwd)



"""
Manage your Anaconda Server server-side settings

###### Set server-side settings

You can add a site named **site_name** like this:

    conda repo admin --set SETTING_NAME SETTING_VALUE

Currently supported settings are:

    - `user_channel_autocreate` - if True, a user's channel is created automatically on first login

###### Show server-side settings

Yuo can see current settings:

    conda repo admin --show

###### Set anaconda ident settings

You can set anaconda ident settings:

    conda repo admin --ident_enabled
    or
    conda repo admin --ident_disabled

    You can set anaconda ident settings with starting date and tokens:
    conda repo admin --ident-enforce-date 2020-01-01 --ident-tokens client_token,session_token,environment_token,username,hostname,environment,organization

"""
import logging
from argparse import RawDescriptionHelpFormatter

from .. import errors
from ..utils.format import SettingsFormatter
from ..utils.yaml import safe_load
from .base import SubCommandBase

logger = logging.getLogger("repo_cli")


SETTINGS_VALIDATOR = {"user_channel_autocreate": safe_load}


class SubCommand(SubCommandBase):
    name = "admin"

    def main(self):
        if self.args.show:
            self.show_settings()
            return

        if self.args.get:
            self.show_settings(self.args.get)
            return

        if self.args.set:
            self.update_settings(self.args.set)
            return

        if (
            self.args.ident_enabled
            or self.args.ident_disabled
            or self.args.ident_enforce_date
            or self.args.ident_tokens
        ):
            enabled = True if self.args.ident_enabled else None
            if self.args.ident_disabled:
                enabled = False

            self.update_conda_ident_settings(
                enabled,
                self.args.ident_enforce_date if self.args.ident_enforce_date else None,
                self.args.ident_tokens if self.args.ident_tokens else None,
            )
            return

        raise NotImplementedError("Please use command options")

    def show_settings(self, key=None):
        settings = self.api.get_system_settings()
        if key is not None:
            if key not in settings:
                raise errors.RepoCLIError("%s is an unknown admin setting" % key)
            settings = {key: settings[key]}

        self.log.info(SettingsFormatter.format_object_as_list(settings))

    def update_settings(self, args):
        data = {}
        for key, value in args:
            if key not in SETTINGS_VALIDATOR:
                raise errors.RepoCLIError("%s is an unknown admin setting" % key)
            data[key] = SETTINGS_VALIDATOR[key](value)

        settings = self.api.get_system_settings()
        settings.update(data)
        self.api.update_system_settings(settings)

        self.log.info("Anaconda Server settings are updated")

    def update_conda_ident_settings(self, enabled, date=None, tokens_serialize=None):
        settings = self.api.get_system_settings()

        ident_settings = settings.get("anaconda_ident_settings", {})
        if not ident_settings:
            ident_settings = {}

        if enabled is not None:
            ident_settings["enabled"] = enabled
        if date:
            ident_settings["enable_from"] = date
        if tokens_serialize:
            tokens = tokens_serialize.split(",")
            for token in tokens:
                if token not in [
                    "client_token",
                    "session_token",
                    "environment_token",
                    "username",
                    "hostname",
                    "environment",
                    "organization",
                ]:
                    raise errors.RepoCLIError("Invalid token: %s" % token)
            ident_settings["selected_tokens"] = tokens_serialize.split(",")

        settings["anaconda_ident_settings"] = ident_settings

        self.api.update_system_settings(settings)
        self.log.info("Anaconda Ident settings are updated")

    def add_parser(self, subparsers):
        description = "Anaconda Server admin settings"
        parser = subparsers.add_parser(
            "admin",
            help=description,
            description=description,
            epilog=__doc__,
            formatter_class=RawDescriptionHelpFormatter,
        )

        agroup = parser.add_argument_group("actions")

        agroup.add_argument(
            "--set",
            nargs=2,
            action="append",
            default=[],
            help="sets a server setting value: name value",
            metavar=("name", "value"),
        )
        agroup.add_argument("--get", metavar="name", help="get value: name")
        agroup.add_argument(
            "--show", action="store_true", default=False, help="show all variables"
        )

        agroup.add_argument(
            "--ident_enabled",
            action="store_true",
            default=True,
            help="enable anaconda ident",
        )
        agroup.add_argument(
            "--ident_disabled",
            action="store_true",
            default=False,
            help="disable anaconda ident",
        )
        agroup.add_argument(
            "--ident-enforce-date",
            metavar="date",
            help="set anaconda ident date from when conda ident is mandatory",
        )
        agroup.add_argument(
            "--ident_tokens",
            metavar="tokens",
            help="set comma seperated list of tokens for anaconda ident. Must be one of: client_token, session_token, environment_token, username, hostname, environment, organization]",
        )

        parser.set_defaults(main=self.main, sub_parser=parser)
